using System;
using System.IO;
using System.IO.Compression;
using System.Collections.Generic;
using System.Text.RegularExpressions;

/*
Contains: 18 methods as of version 09 on 16th Sept. 2018

CompressDirectoryToZipArchive()
CopyTo()
CountDirectories()
DeleteThisFile()
DeleteThisFile() overload that takes filepath and second arg specifies what extension must be; added in v9
DirectoryContainsTheseFiles()
DirectoryDepth() added in v8
ExtractArchiveToDirectory()
GetDirectoryFileCount()
GetFilesRecursively()
GetPathPortionOfFullPath() added in v8
GetTopLevelFileCount()
GetUsernameAndEmail() added in v9
IsEmail() added in v9
IsValidPath() added in v8
LogToTextFile() added in v8
ProcessDirectoryForFileDeletions()
ProcessDirectoryForFileDeletions() overload where second arg specifies extension of files to be deleted; added in v9
*/

namespace com.wms.io{
	
	public class MyDirAndFile{	
	
		public const string MatchEmailPattern =
           @"^[a-zA-Z][\w\.-]*[a-zA-Z0-9]@[a-zA-Z0-9][\w\.-]*[a-zA-Z0-9]\.[a-zA-Z][a-zA-Z\.]*[a-zA-Z]$";
	
		public static bool CompressDirectoryToZipArchive(string targetDirectory)
		{
			bool success = false;
			
			if(Directory.Exists(targetDirectory)){
				int subdirs = CountDirectories(targetDirectory);
				int files = GetDirectoryFileCount(targetDirectory);
				if((subdirs >= 0) && (files > 0)){
					try{
						string dirName = new DirectoryInfo(targetDirectory).Name;
						string fileOut = dirName + ".zip";
						ZipFile.CreateFromDirectory(targetDirectory, fileOut);
					}catch(Exception exWhileTryingToCompressToZip){
						Console.WriteLine(" Error in CompressDirectoryToZipArchive() method:");
						Console.WriteLine(" " + exWhileTryingToCompressToZip.Message);
					}
				}
			}
			
			return success;
		}

		public static bool CopyTo(string targetDirectory, string destinationDirectory)
		{
			bool blnSuccess = false;
			
			try{
				if(Directory.Exists(targetDirectory)){
					foreach (string dirPath in Directory.GetDirectories(targetDirectory, "*", 
						SearchOption.AllDirectories)){
						Directory.CreateDirectory(dirPath.Replace(targetDirectory, destinationDirectory));	
					}
					//Copy all the files & Replaces any files with the same name
					foreach (string newPath in Directory.GetFiles(targetDirectory, "*.*", 
						SearchOption.AllDirectories)){
						File.Copy(newPath, newPath.Replace(targetDirectory, destinationDirectory), true);
						blnSuccess = true;
					}						
				}				
			}catch(Exception exCopyTo){
				Console.WriteLine("Error in ioLibrary: " + exCopyTo.Message);
			}
			
			return blnSuccess;
			
		}
		
		public static int CountDirectories(string targetDirectory)
		{
			int iDirs = 0;
			
			if(Directory.Exists(targetDirectory)){
				iDirs = Directory.GetDirectories(targetDirectory, "*", SearchOption.AllDirectories).Length;
			}
			
			return iDirs;
		}
		
        public static bool DeleteThisFile(string path) 
        {           
			bool blnDeletionOccurred = false;
			
			FileInfo fi = new FileInfo(path);
			
			if(File.Exists(path)){
				try{								
					File.Delete(fi.FullName);
					blnDeletionOccurred = true;										
				}catch(Exception exWhileDeleting){
					Console.WriteLine(" Error while attempting to delete {0}", fi.FullName);
					Console.WriteLine(" " + exWhileDeleting.Message);
				}
			}

			return blnDeletionOccurred;
        }	

		public static bool DeleteThisFile(string path, string fileExtension)
		{
			bool blnDeletionOccurred = false;
			
			FileInfo fi = new FileInfo(path);
			
			if(File.Exists(path)){
				if(path.EndsWith(fileExtension)){
					try{								
						File.Delete(fi.FullName);
						blnDeletionOccurred = true;										
					}catch(Exception exWhileDeleting){
						Console.WriteLine(" Error while attempting to delete {0}", fi.FullName);
						Console.WriteLine(" " + exWhileDeleting.Message);
					}
				}
			}

			return blnDeletionOccurred;			
		}

		public static bool DirectoryContainsTheseFiles(string targetDirectory, List<string> _fileList)
		{
			bool result = false;
			int sought_count = _fileList.Count;
			if(Directory.Exists(targetDirectory)){
				string[] files = Directory.GetFiles(targetDirectory);
				int iCount = 0;
				foreach(string p in _fileList){
					for(int i=0; i < files.Length; i++){
						string sName = Path.GetFileName(files[i]);
						if(sName == p){ iCount++; }
					}
				}
				if(iCount == sought_count){ result = true; }
			}
			
			return result;
		}		
		
		public static int DirectoryDepth(string someValidPath)
		{
			int depth = -1;			
			
			if(!IsValidPath(someValidPath)){ 
				return depth; 
			}else{
				Console.Write(" Calculating directory depth... ");
				char[] splitter = new char[]{'\\'};
				string[] tokens = someValidPath.Split(splitter, StringSplitOptions.RemoveEmptyEntries);
				Console.Write("{0} deep\n", tokens.Length);
				
				for(int i=0; i < tokens.Length; i++){
					if(i % 2 == 0 && (i != 0)){
						Console.WriteLine("\tDepth {0}: {1}; ", (i + 1), tokens[i]);
					}else{
						Console.Write("\tDepth {0}: {1}; ", (i + 1), tokens[i]);
					}
				}
				Console.WriteLine("");
				
			}
						
			return depth;
		}
		
		public static bool ExtractArchiveToDirectory(string archive, string targetDirectory)
		{
			bool success = false;
			
			if(File.Exists(archive)){
				if(!Directory.Exists(targetDirectory)){
					try{
						Directory.CreateDirectory(targetDirectory);
					}catch(Exception exAttemptingCreateDir){
						Console.WriteLine("Error while attempting to create directory for zip extraction...");
						Console.WriteLine(exAttemptingCreateDir.Message);
					}
				}
				try{
					ZipFile.ExtractToDirectory(archive, targetDirectory);
				}catch(Exception exWhileExtracting){
					Console.WriteLine("Error during zip extraction...");
					Console.WriteLine(exWhileExtracting.Message);
				}
			}
			
			return success;
		}		
		
		public static int GetDirectoryFileCount(string path)
		{
			int fileCount = 0;
			
			if(Directory.Exists(path))
			{
				fileCount = Directory.GetFiles(path, "*.*", SearchOption.AllDirectories).Length;
			}
			
			return fileCount;
		}
		
		public static List<string> GetFilesRecursively(string b) {

            // 1.Store results in the file results list.
            List<string> result = new List<string>();

            // 2.Store a stack of our directories.
            Stack<string> stack = new Stack<string>();

            // 3.Add initial directory.
            stack.Push(b);

            // 4.Continue while there are directories to process
            while (stack.Count > 0) {
                // A.Get top directory
                string dir = stack.Pop();

                try {
                    // B. Add all files at this directory to the result List.
                    result.AddRange(Directory.GetFiles(dir, "*.*"));

                    // C. Add all directories at this directory.
                    foreach (string dn in Directory.GetDirectories(dir)) {
                        stack.Push(dn);
                    }
                }
                catch {
                    // D. Could not open the directory
                }
            }
            return result;
        }				
		
		public static string GetPathPortionOfFullPath(string fullpath)
		{
			//be sure to test fullpath with IsValidPath() before passing in parameter...
			string sFilename = Path.GetFileName(fullpath);
			int index = fullpath.IndexOf(sFilename);
			string result = fullpath.Substring(0, index);
			return result;
		}

		public static int GetTopLevelFileCount(string path)
		{
			int fileCount = 0;
			
			if(Directory.Exists(path))
			{
				fileCount = Directory.GetFiles(path, "*.*", SearchOption.TopDirectoryOnly).Length;
			}
			
			return fileCount;
		}
		
		public static string GetUsernameAndEmail()
		{
			string sName = string.Empty;
			string sEmail = string.Empty;
			string sFirstname = string.Empty;
			string sLastname = string.Empty;
			string sConfirm = "N";
			
			while(sConfirm.Trim().ToUpper().Substring(0,1) != "Y"){
				Console.Write("\n What is your first name? ");
				sName = Console.ReadLine().Trim().ToLower();
				
				if(sName.Length >= 2){					
					sFirstname = sName.Substring(0,1).ToUpper() + sName.Substring(1, sName.Length -1);
					Console.Write("Your first name is \"" + sFirstname + "\". Did I get that right? Y/N >");
					sConfirm = Console.ReadLine();
					if(sConfirm.Length < 1){
						sConfirm = "N";
					}else{
						sConfirm = sConfirm.Substring(0,1).ToUpper();
					}
				}else{
					Console.Write("Shortest acceptable first name is two characters; ex: \"Bo\"");
					sConfirm = "N";
				}												
			}
			
			Console.WriteLine("Thank you for providing your first name, {0}", sFirstname);
			
			sConfirm = "N";
			
			while(sConfirm.Trim().ToUpper().Substring(0,1) != "Y"){
				Console.Write("\n What is your last name? ");
				sName = Console.ReadLine().Trim().ToLower();
				
				if(sName.Length >= 2){					
					sLastname = sName.Substring(0,1).ToUpper() + sName.Substring(1, sName.Length -1);
					Console.Write(" Your last name is \"" + sLastname + "\". Did I get that right? Y/N >");
					sConfirm = Console.ReadLine().Trim().ToUpper();
					if(sConfirm.Length < 1){
						sConfirm = "N";
					}else{
						sConfirm = sConfirm.Substring(0,1).ToUpper();
					}
				}else{
					Console.Write(" Shortest acceptable last name is two characters; ex: \"Yu\"");
					sConfirm = "N";
				}												
			}			

			Console.WriteLine("Thank you for providing last name \"{0}\"", sLastname);
			
			sConfirm = "N";
			
			while(sConfirm.Trim().ToUpper().Substring(0,1) != "Y"){
				Console.Write("\n What is your email address? ");
				sEmail = Console.ReadLine().Trim();
				
				if(sEmail.Length >= 6){					
					if(IsEmail(sEmail)){
						Console.Write(" You entered \"{0}\". Is that your correct\n", sEmail);
						Console.Write(" email address? Y/N >", sEmail);
						sConfirm = Console.ReadLine().Trim().ToUpper();
						if(sConfirm.Length < 1){
							sConfirm = "N";
						}else{
							sConfirm = sConfirm.Substring(0,1).ToUpper();
						}
					}else{
						Console.WriteLine(" \"{0}\" is not a valid email address...", sEmail);
					}
				}else{
					Console.Write(" The shortest acceptable email address is six characters.");
					sConfirm = "N";
				}												
			}						
			
			Console.WriteLine(" Thank you for providing your email address.");
			
			sConfirm = string.Empty;
			
			string result = "(" + sFirstname + " " + sLastname + " <" + sEmail + ">)";
			Console.WriteLine(" Summary: {0}\n", result);
			string sNameEmailPath = "C:\\csdev\\user\\nameemail.txt";
			if(!File.Exists(sNameEmailPath)){
				/* while((sConfirm != "Y") && (sConfirm!="N")){
					Console.Write(" Do you consent to saving your name and email to disk? Y/N >");
					sConfirm = Console.ReadLine().Trim().ToUpper();
					if(sConfirm.Length < 1){
						sConfirm = "N";
					}else{
						sConfirm = sConfirm.Substring(0,1).ToUpper();
					}
				} */
/* 				if(sConfirm == "Y"){
					try{
						if(!Directory.Exists("C:\\csdev")){ Directory.CreateDirectory("C:\\csdev");}
						if(!Directory.Exists("C:\\csdev\\user")){Directory.CreateDirectory("C:\\csdev\\user");}
						File.WriteAllText(sNameEmailPath, result);
						Console.WriteLine(" Successfully saved name and email to file at:");
						Console.WriteLine(" " + sNameEmailPath + "\n");
					}catch(Exception exWhileWritingNameEmail){
						Console.WriteLine(" Error while attempting to write name/email to file via ioLibrary.cs");
						Console.WriteLine(" " + exWhileWritingNameEmail.Message);
						Console.WriteLine("\"" + sNameEmailPath + "\"");
					}
				} */
				try{
					if(!Directory.Exists("C:\\csdev")){ Directory.CreateDirectory("C:\\csdev");}
					if(!Directory.Exists("C:\\csdev\\user")){Directory.CreateDirectory("C:\\csdev\\user");}
					File.WriteAllText(sNameEmailPath, result);
					Console.WriteLine(" Successfully saved name and email to file at:");
					Console.WriteLine(" " + sNameEmailPath + "\n");
				}catch(Exception exWhileWritingNameEmail){
					Console.WriteLine(" Error while attempting to write name/email to file via ioLibrary.cs");
					Console.WriteLine(" " + exWhileWritingNameEmail.Message);
					Console.WriteLine("\"" + sNameEmailPath + "\"");
				}				
			}
			return result;
		}
		
        public static bool IsEmail(string sEmail) {
            if (sEmail != null) {
                return Regex.IsMatch(sEmail, MatchEmailPattern);
            }
            else {
                //if sEmail is null, we don't test at all
                return false;
            }
        }				
				
		public static bool IsValidPath(string path, bool allowRelativePaths = false)
		{
			bool isValid = true;

			try
			{
				string fullPath = Path.GetFullPath(path);

				if (allowRelativePaths)
				{
					isValid = Path.IsPathRooted(path);
				}
				else
				{
					string root = Path.GetPathRoot(path);
					isValid = string.IsNullOrEmpty(root.Trim(new char[] { '\\', '/' })) == false;
				}
			}
			catch(Exception ex)
			{
				isValid = false;
				LogToTextFile(ex.Message, "log.txt");
			}

			return isValid;
		}

		public static void LogToTextFile(string message, string path)
		{
			try{
				File.WriteAllText(path, message);
			}catch(Exception exWhileLoggingVia_LogToTextFile){
				Console.WriteLine("Exception thrown by LogToTextFile in ioLibrary.cs:");
				Console.WriteLine(exWhileLoggingVia_LogToTextFile.Message);
			}
		}
		
        public static void ProcessDirectoryForFileDeletions(string targetDirectory) 
        {
			// Process the list of files found in the directory.
            string [] fileEntries = Directory.GetFiles(targetDirectory);
            foreach(string fileName in fileEntries){
                bool result = DeleteThisFile(fileName);				
			}
 
            // Recurse into subdirectories of this directory.
            string [] subdirectoryEntries = Directory.GetDirectories(targetDirectory);
            foreach(string subdirectory in subdirectoryEntries){
                ProcessDirectoryForFileDeletions(subdirectory); 
			}				
        }
		
		public static void ProcessDirectoryForFileDeletions(string targetDirectory, string fileExtensionToDelete)
		{
			// Process the list of files found in the directory.
            string [] fileEntries = Directory.GetFiles(targetDirectory);
            foreach(string fileName in fileEntries){
                bool result = DeleteThisFile(fileName, fileExtensionToDelete);				
			}
 
            // Recurse into subdirectories of this directory.
            string [] subdirectoryEntries = Directory.GetDirectories(targetDirectory);
            foreach(string subdirectory in subdirectoryEntries){
                ProcessDirectoryForFileDeletions(subdirectory); 
			}				
			
		}
		
         
    }		
		
	
	
	
}